#pragma once
#include "../../../internal-include/GPU/gpu_internal.hpp"
#include "bios_font_types.hpp"
#include <PSX/System/syscalls.hpp>

// | !    | "    | #    | $    | %    | &    | '    | (    | )    | *    | +    | ,    | -    | .    | /    | 0    |
// | 8149 | 8168 | 8194 | 8190 | 8193 | 8195 | 8166 | 8169 | 816A | 8196 | 817B | 8143 | 817C | 8144 | 815E | 8240 |
// | 1    | 2    | 3    | 4    | 5    | 6    | 7    | 8    | 9    | :    | ;    | <    | =    | >    | ?    | @    |
// | 8251 | 8252 | 8253 | 8254 | 8255 | 8256 | 8257 | 8258 | 8259 | 8146 | 8147 | 8183 | 8181 | 8184 | 8148 | 8197 |
// | A    | B    | C    | D    | E    | F    | G    | H    | I    | J    | K    | L    | M    | N    | O    | P    |
// | 8260 | 8261 | 8262 | 8263 | 8264 | 8265 | 8266 | 8267 | 8268 | 8269 | 826A | 826B | 826C | 826D | 826E | 826F |
// | Q    | R    | S    | T    | U    | V    | W    | X    | Y    | Z    | [    | \    | ]    | ^    | _    | `    |
// | 8270 | 8271 | 8272 | 8273 | 8274 | 8275 | 8276 | 8277 | 8278 | 8279 | 816D | 815F | 816E | 814F | 8151 | 814D |
// | a    | b    | c    | d    | e    | f    | g    | h    | i    | j    | k    | l    | m    | n    | o    | p    |
// | 8281 | 8282 | 8283 | 8284 | 8285 | 8286 | 8287 | 8288 | 8289 | 828A | 828B | 828C | 828D | 828E | 828F | 8290 |
// | q    | r    | s    | t    | u    | v    | w    | x    | y    | z    | {    | |    | }    | ~    |      |      | 
// | 8291 | 8292 | 8293 | 8294 | 8295 | 8296 | 8297 | 8298 | 8299 | 829A | 816F | 8162 | 8170 | 8160 |      |      |

namespace JabyEngine {
    namespace GPU {
        namespace SJIS {
            // base: 0x8100
            static const SpecialChar Specials[] = { // ToDo: Can we split this into 4 arrays? Would that help somehow?
                // {   !   }  {   "   }  {   #   }  {   $   }  {   %   }  {   &   }  {   '   }  {   (   }  {   )   }  {   *   }  {    +   }  {    ,   }  {    -   }  {    .   }  {    /   }
                {0x49, 0}, {0x68, 1}, {0x94, 2}, {0x90, 3}, {0x93, 4}, {0x95, 5}, {0x66, 6}, {0x69, 7}, {0x6A, 8}, {0x96, 9}, {0x7B, 10}, {0x43, 11}, {0x7C, 12}, {0x44, 13}, {0x5E, 14},
                // {    :   }  {    ;   }  {    <   }  {    =   }  {    >   }  {    ?   }  {    @   }
                {0x46, 25}, {0x47, 26}, {0x83, 27}, {0x81, 28}, {0x84, 29}, {0x48, 30}, {0x97, 31},
                // {    [   }  {    \   }  {    ]   }  {    ^   }  {    _   }  {    `   }   
                {0x6D, 58}, {0x5F, 59}, {0x6E, 60}, {0x4F, 61}, {0x51, 62}, {0x4D, 63},
                // {    {   }  {    |   }  {    }   }  {    ~   }   
                {0x6F, 90}, {0x62, 91}, {0x70, 92}, {0x60, 93}
            };

            // base: 0x8200
            static const RangeChar AlphaNumeric[] = {
            //  {       0      }
                {{0x4F, 15},  1},
            //  {     1 - 9    }
                {{0x50, 16},  9},
            //  {     A - Z    }
                {{0x60, 32}, 26},
            //  {     a - z    }
                {{0x81, 64}, 26}
            };

            static void load(const PositionU16& start_pos) {
                FontBuffer font_buffer;
                const auto load_special_chars = [&font_buffer]<size_t Size>(const PositionU16& pos, const SpecialChar(&special_chars)[Size]) {
                    for(const auto& special_char : special_chars) {
                        font_buffer.load_to(pos.add(FontBuffer::vram_offset(special_char.tile_id)), SysCall::Krom2RawAdd(0x8100 | special_char.base_offset));
                    }
                };
                const auto load_alpha_num = [&font_buffer]<size_t Size>(const PositionU16& pos, const RangeChar(&range_char)[Size]) {
                    for(const auto& range : range_char) {
                        const auto end_tile  = range.start_char.tile_id + range.length;
                        auto       sjis_code = 0x8200 | range.start_char.base_offset;

                        for(uint16_t tile_id = range.start_char.tile_id; tile_id < end_tile; tile_id++,sjis_code++) {
                            font_buffer.load_to(pos.add(FontBuffer::vram_offset(tile_id)), SysCall::Krom2RawAdd(sjis_code));
                        }
                    }
                };

                font_buffer.setup();

                GPU::internal::DMA::Receive::prepare();
                load_special_chars(start_pos, Specials);
                load_alpha_num(start_pos, AlphaNumeric);
                font_buffer.shutdown();
            }
        }
    }
}