mod file_tab;
mod gui_elements;
mod main_tab;

use crate::MainWindow;
use rfd::MessageDialog;
use std::{cell::RefCell, rc::Rc};
use slint::{Rgba8Pixel, SharedPixelBuffer};
use tiny_skia::{Rect, Transform};

pub use gui_elements::GUIElements;

pub const VRAM_WIDTH:usize  = 1024;
pub const VRAM_HEIGHT:usize = 512;

type MainWindowRef  = Rc<RefCell<MainWindow>>;
type GUIElementsRef = Rc<RefCell<GUIElements>>;

pub fn display_information(title: impl Into<String>, text: impl Into<String>) {
    MessageDialog::new().set_title(title).set_level(rfd::MessageLevel::Info).set_description(text).show();
}

pub fn display_error(title: impl Into<String>, text: impl Into<String>) {
    MessageDialog::new().set_title(title).set_level(rfd::MessageLevel::Error).set_description(text).show();
}

fn create_vram_bg(rect_width: u32, rect_height: u32) -> Option<SharedPixelBuffer<Rgba8Pixel>> {
    let mut pixel_buffer = SharedPixelBuffer::<Rgba8Pixel>::new(VRAM_WIDTH as u32, VRAM_HEIGHT as u32);
    let     width        = pixel_buffer.width();
    let     height       = pixel_buffer.height();

    if let Some(mut pixmap) = tiny_skia::PixmapMut::from_bytes(pixel_buffer.make_mut_bytes(), width, height) {
        let vram_color           = tiny_skia::Color::from_rgba8(0x0,  0x80, 0x80, 0xFF); 
        let display_buffer_color = tiny_skia::Color::from_rgba8(0x0,  0x80,  0x0, 0xFF);
        let draw_area_color      = tiny_skia::Color::from_rgba8(0x80, 0x80,  0x0, 0xFF);

        let mut paint = tiny_skia::Paint::default();

        pixmap.fill(vram_color);
        
        paint.set_color(display_buffer_color);
        pixmap.fill_rect(Rect::from_xywh(0.0, 0.0,                rect_width as f32, rect_height as f32).unwrap(), &paint, Transform::identity(), None);
        paint.set_color(draw_area_color);
        pixmap.fill_rect(Rect::from_xywh(0.0, rect_height as f32, rect_width as f32, rect_height as f32).unwrap(), &paint, Transform::identity(), None);
        Some(pixel_buffer)
    }
    
    else {
        None
    }
}