pub mod tim;
use std::{ops::RangeInclusive, path::PathBuf};
use slint::Image;
use tim::{types::Encoding, TIMInfo};
use tool_helper::Error;

pub struct Logic {
    added_tims:  Vec<Option<TIMInfo>>,
    unadded_tim: Option<TIMInfo>,
}

impl Logic {
    pub fn new() -> Logic {
        Logic{added_tims: Default::default(), unadded_tim: None}
    }

    pub fn remove_added_tim(&mut self, range: RangeInclusive<usize>) {
        let idx = *range.start();
        for _ in range {
            self.added_tims.remove(idx);
        }
    }

    pub fn load_unadded_tim(&mut self, path: &PathBuf) -> Result<(Image, Option<Image>), Error> {
        let tim_info = TIMInfo::from_image(path)?;
        let image    = tim_info.get_slint_images(Encoding::FullColor);

        self.unadded_tim = Some(tim_info);
        Ok(image)
    }

    pub fn get_converted_unadded_tim_image(&self, encoding: Encoding) -> Result<(Image, Option<Image>), Error> {
        if let Some(unadded_tim) = &self.unadded_tim {
            let (image, palette) = unadded_tim.get_slint_images(encoding);
            Ok((image, palette))
        }

        else {
            Err(Error::from_str("No data found for loaded image"))
        }
    } 

    pub fn add_unadded_tim(&mut self, images_added: usize) -> Result<(), Error> {
        if let Some(_) = &self.unadded_tim {
            if images_added >= 1 {
                self.added_tims.push(std::mem::replace(&mut self.unadded_tim, None));

                for _ in 0..(images_added - 1) {
                    self.added_tims.push(None);
                }
                Ok(())
            }

            else {
                Err(Error::from_str("Can not add 0 images"))
            }
        }

        else {
            Err(Error::from_str("No data found for loaded image"))
        }
    }

    pub fn change_unadded_tim_palette_size(&mut self, width: u32, height: u32) -> Result<Option<Image>, Error> {
        if let Some(unadded_tim) = &mut self.unadded_tim {
            unadded_tim.change_palette_size(width, height)
        }

        else {
            Ok(None)
        }
    }
}